package org.leedberg.cotiro;


/*
    Cotiro 0.6.5
	College Time and Room Scheduler Using Constraint Satisfaction
    Copyright (C) 2003 - 2007, Gregory G. Leedberg

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public abstract class Constraint 
{

	/**
	 * 
	 */
	// Most constraints are binary, involving two objects.  So, we will
	// provide storage for these two objects at this abstract level
	public Object A;
   	public Object B;
   	
   	// True if this is a soft constraint, i.e., we should make a best effort
   	// to satisfy it, but can violate it if necessary
   	public boolean isSoft;
   	
   	// The preferred way to implement this in children is with the line:
   	// super(course, building, (SoftSetting)s.useSoft(DEFAULT_BOOLEAN));   	
    
    protected Constraint(Object A, Object B, boolean isSoft) {
    	this.A = A;
    	this.B = B;
    	
    	this.isSoft = isSoft;
    }
    
	// If this constraint is unary, this will return the variable the constraint 
	// involves
	public Object getVarA() {
			return A;
	}
	
	public Object getVarB() {
		return B;
	}
	// Returns true if this constraint is satisfied with the assignments in vars
	// Each type of constraint will implment this function
    public abstract boolean satisfied(VariableList vars);   
    
    // Returns a string representation of the constraint
    public abstract String toString();  
    
    
    // The following two methods are STATIC because they apply to 
    // a type of constraint, not to a specific constraint.  
    // They really should be abstract but Java won't allow
    // abstract static classes, so we have default
    // functionality present.
    
    // Return true if this constraint is unary (does not depend on other variables)
    public boolean isUnary() { return false; }
    
}
