package org.leedberg.cotiro;
import java.util.Iterator;
import java.util.Vector;

/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
public class Constraints 
{
	protected Vector allConstraints; 

	/**
	 * 
	 */
	public Constraints() {
		super();
		allConstraints = new Vector();
	}
	
	// Check each constraint for being satisfied.  As soon as we
	// find one not satisfied, we can quit
	public boolean satisfied(State state) {
		Iterator i = allConstraints.iterator();
		
		while (i.hasNext()) {
			Constraint c = (Constraint)i.next();
			if (!c.satisfied(state.getAssignedVars())) {
				CotiroLogic.println("Violated: " + c);
				return false;
			}
		}
		return true;
	}
	
	// Return the number of constraints stored in this collection which are
	// satisfied by the given state
	// Note that this is necessarily more computationally expensive than
	// regular satisifed() because this must go through ALL constraints in
	// order to count, whereas satisifed only needs to find one
	// constraint that is violated.
	public int numSatisfied(State state) {
		int retVal = 0;
		Iterator i = allConstraints.iterator();
		
		while (i.hasNext()) {
			Constraint c = (Constraint)i.next();
			
			if (c.satisfied(state.getAssignedVars())) {
				retVal++;
			}
		}
		return retVal;
		
	}
	
	public void addConstraint(Constraint constraint) {
		allConstraints.add(constraint);
	}
	
	public Constraint getConstraint(int index) {
		return (Constraint)allConstraints.get(index);
	}
	
	public int size() {
		return allConstraints.size();
	}
	
	// Remove the specified constraint
	public Constraint removeConstraint(int i) {
		return (Constraint)allConstraints.remove(i);
	}
	
	public String toString() {
		String returnVal = "";
		
		for (int i=0; i < allConstraints.size(); i++) {
			Constraint c = (Constraint)allConstraints.get(i);
			returnVal = returnVal + c.toString() + "\n";
		}
		
		return returnVal;
	}
	
	public void println() {		
		for (int i=0; i < allConstraints.size(); i++) {
			Constraint c = (Constraint)allConstraints.get(i);
			CotiroLogic.println(c.toString());
		}	

	}
}
