package org.leedberg.cotiro;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

public class Time implements Cloneable
{
	protected int hour;
	protected int minute;

	/**
	 * 
	 */
	public Time() {
		super();
		hour = 0;
		minute = 0;
	}
	
	// Constructor with specified hour and minute
	public Time(int hour, int minute) {
		this.hour = hour;
		this.minute = minute;
	}
	
	// Constructor with string in the format HH:MM or H:MM
	public Time(String time) {
		int colon = time.indexOf(":");
		
		if (colon == -1) {
			throw new IllegalArgumentException();
		}
		
		String hour = time.substring(0,colon);
		String minute = time.substring(colon+1, time.length());
		
		this.hour = (new Integer(hour)).intValue();
		this.minute = (new Integer(minute)).intValue();		
	}
        
        public static boolean isValid(String time) {
            // Returns true if this is a valid time string
            // Check for one colon
            if (time.indexOf(":") == -1) {
                return false;
            }
            
            if (time.indexOf(":") != time.lastIndexOf(":")) {
                return false;
            }
            
            // Parse hour and minute
            Time parsedTime;
            
            try {
                parsedTime = new Time(time);
            } catch (Exception e) {
                return false;
            }
            
            // Check for hour <= 24
            if (parsedTime.hour > 24) {
                return false;
            }
            
            // Check for minute <= 60
            if (parsedTime.minute > 59) {
                return false;
            }
            
            return true;
        }
	
	public boolean equals(Time t) {
		return (hour == t.hour && minute == t.minute);
	}
	
	public boolean earlierThan ( Time t) {
		return ((hour < t.hour) || (hour == t.hour && (minute < t.minute)));
	}

	public boolean laterThan ( Time t) {
		return ((hour > t.hour) || (hour == t.hour && (minute > t.minute)));
	}
	public int minus(Time t) {
		// convert both times into a number of minutes since midnight
		int thisTime = (this.hour * 60) + this.minute;
		int otherTime = (t.hour * 60) + t.minute;		
		
		return thisTime - otherTime;
	}
	
	public String toString() {
		if (minute >= 10)  return hour + ":" + minute;
		else return hour + ":0" + minute;
	}
	
	public void addMinutes(int amount) {
		minute += amount;
		
		hour += minute / 60;
		
		minute %= 60;
	}
        
        public int getHour24() {
            return this.hour;
        }
        
        public int getHour12() {
            return hour > 12 ? hour - 12: hour;
        }
        
        public String getAMPM() {
            return hour >= 12 ? "PM" : "AM";
        }
        
        public String getMinute() {
            return (minute < 10 ? "0" : "") + this.minute;
        }
        

	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException s) {
			return null; 
		}
	}
}
