package org.leedberg.cotiro;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
public class TimeSpan implements Cloneable
{
	protected Time start, end;
	protected Days days;

	public TimeSpan() {
		super();
	}
	
	public TimeSpan(Time start, Time end, Days days) {
		this.start = start;
		this.end = end;		
		this.days = days;
	}
	
	public Days getDays() {
		return days;
	}
	
	public Time getStart() {
		return start;
	}
	
	public Time getEnd() {
		return end;
	}
	

	
	// Returns true if the timespan overlaps with span
	public boolean overlap ( TimeSpan span) {
		return  (span.days.overlaps(days) && 
				((span.start.laterThan(start) && span.start.earlierThan(end)) ||
					(span.end.laterThan(start) && span.end.earlierThan(end)) ||
					(span.start.equals(start)) ||
					(span.start.equals(end)) ||
					(span.end.equals(start)) ||
					(span.end.equals(end))));
	}
	
	// Returns true if this timespan encompasses the supplied span
	public boolean encompasses( TimeSpan span ) {
		return (days.encompasses(span.days) &&
				(span.start.laterThan(start) || (span.start.equals(start))) &&
					(span.end.earlierThan(end) || (span.end.equals(end))));
	}
	
	// Returns true if this timespan encompasses the supplied span
	public boolean encompassesIgnoreDays( TimeSpan span ) {
		return ((span.start.laterThan(start) || (span.start.equals(start))) &&
					(span.end.earlierThan(end) || (span.end.equals(end))));
	}

	
	public String toString() {
		return start + " - " + end + " (" + days + ")";
	}
	
	public Object clone() {
		TimeSpan copy;
		
		try {		
			copy = (TimeSpan)super.clone();
			copy.start = (Time)start.clone();
			copy.end = (Time)end.clone();
			return copy;
		} catch (CloneNotSupportedException s) {
			return null;
		}
	}

}
