package org.leedberg.cotiro.college;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlOptions;
import org.leedberg.cotiro.CotiroLogic;
import org.leedberg.cotiro.CourseScheduleDocument;
import org.leedberg.cotiro.CourseScheduleDocument.CourseSchedule.Course;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
public class BrowseSchedule {

	private enum OutputFile {
		Text, XML, HTML
	};

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		OutputFile outputType = OutputFile.Text;
		String inputFile = null;
		String outputFile = null;

		if (args.length < 1) {
			// There should always be at least 1 argument:
			// inputfile
			BrowseSchedule.outputHelp();
			System.exit(-1);
		}

		inputFile = args[0];
		System.out.println("Input file: " + inputFile);
		int i;
		for (i = 1; i < args.length; i++) {

			if (args[i].equals("-t")) {
				System.out.println("Text output");
				outputType = OutputFile.Text;
			} else if (args[i].equals("-x")) {
				System.out.println("XML output");
				outputType = OutputFile.XML;
			} else if (args[i].equals("-h")) {
				System.out.println("HTML output");
				outputType = OutputFile.HTML;
			} else {
				if (outputFile == null) {
					outputFile = args[i];
				} else {
					BrowseSchedule.outputHelp();
					System.exit(-1);
				}
			}
		}

		if (inputFile == null) {
			CotiroLogic.println("ERROR: Input file not specified!");
			BrowseSchedule.outputHelp();			
			System.exit(-1);
		}

		// Build up an XPath search query
		BufferedReader userIn = new BufferedReader(new InputStreamReader(
				System.in));
		String inputLine = "";

		String searchQuery = "";

		try {
			System.out
					.println("Enter search terms below.  To not search on a field, press return.");
			System.out.print("Course=");
			inputLine = userIn.readLine();

			// Some of this code is tough to read... basically, if this query
			// has been typed in and is the
			// first query entered, set searchQuery to it. Otherwise, add it to
			// the search query,
			// but also put an 'and' in the middle of the terms.
			if (!inputLine.equals("")) {
				searchQuery = "ns:name='" + inputLine + "'";
			}
                        // TODO: This query needs to be built differently since the schema changed!!!
			System.out.print("Teacher=");
			inputLine = userIn.readLine();

			if (!inputLine.equals("")) {
				searchQuery = searchQuery
						+ (searchQuery.equals("") ? "" : " and ")
						+ "ns:teacher='" + inputLine + "'";
			}

			System.out.print("Building=");
			inputLine = userIn.readLine();

			if (!inputLine.equals("")) {
				searchQuery = searchQuery
						+ (searchQuery.equals("") ? "" : " and ")
						+ "ns:building='" + inputLine + "'";
			}

			System.out.print("Room=");
			inputLine = userIn.readLine();

			if (!inputLine.equals("")) {
				searchQuery = searchQuery
						+ (searchQuery.equals("") ? "" : " and ") + "ns:room='"
						+ inputLine + "'";
			}

		} catch (IOException e) {
			System.err.println("BrowseSchedule: IOException");
			e.printStackTrace();
			System.exit(-1);
		}

		// Load xml schedule into memory
		CourseScheduleDocument doc = null;

		try {
			doc = CourseScheduleDocument.Factory.parse(new File(inputFile));
		} catch (XmlException e) {
			System.err.println("ERROR: Unable to parse XML input file:");
			System.err.println(e.toString());
			System.exit(-1);
		} catch (IOException e) {
			System.err.println("ERROR: Unable to read file: ");
			System.err.println(e.toString());
			System.exit(-1);
		}

		System.out.println("XPath Search query: " + searchQuery);

		CourseScheduleDocument out = CourseScheduleDocument.Factory
				.newInstance();
		out.addNewCourseSchedule();

		// Perform search

		Course[] courses = null;
		try {
			courses = (Course[]) doc
					.selectPath("declare namespace ns='http://www.leedberg.org/cotiro';$this/ns:courseSchedule/ns:course["
							+ searchQuery + "]");
		} catch (ClassCastException e) {
			// We get this exception if the search yields no results
			courses = new Course[0];
		}

		out.getCourseSchedule().setCourseArray(courses);

		// Use XMLConverter to output results as desired

		//File outFile = null;

		// IF the desitred output is XML, we should just output the schedule to
		// that file
		// Ohterwise, if the desired output is not XML, we should create a
		// temporary XML file for the
		// output, then use XmlConverter to convert that into the desired output
		// format.
		
		File tempFile=null;
		
		try {
			tempFile = File.createTempFile("cotiro","browse");
		} catch (Exception e) {
			System.err.println("Failed to create temp file: " + e.toString());
			System.exit(-1);
		}
		
		try {
			// Save the output to a file
			out.save(tempFile, new XmlOptions().setUseDefaultNamespace());

		} catch (IOException e) {
			e.printStackTrace();
		}
		
		// Convert this XML to text so we can output to the screen
		String textOutput = XmlConverter.convertToText(tempFile);
		
		
		if (outputFile != null) { // If no output file specified, skip this
			/*
			if (outputType.equals(OutputFile.XML)) {
				// Create an actual file
				outFile = new File(outputFile);
			} else {
				// Create a temporary file
				try {
					outFile = File.createTempFile("cotiro", "browse");
				} catch (IOException e) {
					System.err.println("Failed to create temp file: "
							+ e.toString());
					System.exit(-1);
				}

			}

			// At this point, outputFile is an XML file to output the schedule
			// to
			// (either the actual XML output file, or a temp file to convert)
			try {
				// Save the output to a file
				out.save(outFile, new XmlOptions().setUseDefaultNamespace());

			} catch (IOException e) {
				e.printStackTrace();
			}
*/
			// Use the XML file (temporary or otherwise) to produce other
			// desired outputs
			// We're going to cheat and actually call the main method of
			// XmlConverter directly,
			// so that it is really doing all of the work for us. It's arguments
			// are:
			// XmlConverter [-text | -html] [xml_input_file] [output_file]");
			if (outputType.equals(OutputFile.HTML)) {
				// Convert to HTML
				XmlConverter.main(new String[] { "-html",
						tempFile.getAbsolutePath(), outputFile });
				

			} else if (outputType.equals(OutputFile.Text)) {
				// Convert to text
				XmlConverter.main(new String[] { "-text",
						tempFile.getAbsolutePath(), outputFile });				
			} else if (outputType.equals(OutputFile.XML)) {
				try {
					// Save the output to a file
					out.save(new File(outputFile), new XmlOptions().setUseDefaultNamespace());

				} catch (IOException e) {
					e.printStackTrace();
				}
				
			}
		}

		// Output results
		tempFile.delete();
		//System.out.println(out.xmlText());
		System.out.println(textOutput);

	}

	private static void outputHelp() {
		System.out.println("COTIRO BrowseSchedule");
		System.out.println("Usage: browseSchedule inputfile [-t outputfile | -x outputfile | -h outputfile]");
		System.out.println("");
		System.out.println("\tinputfile  Input Cotiro XML file name");

		System.out.println("\t-t  Text output");
		System.out.println("\t-x  XML output");
		System.out.println("\t-h  HTML formatted output");

	}

}
