package org.leedberg.cotiro.college;
import java.math.BigInteger;
import java.util.Properties;
import java.util.Vector;

import org.leedberg.cotiro.CotiroLogic;
import org.leedberg.cotiro.Value;
import org.leedberg.cotiro.Variable;
import org.leedberg.cotiro.CourseScheduleDocument.CourseSchedule.Course;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class CourseSection implements Variable, Comparable {
    public String course;
    public String instructor;
    public int size;
    public TimeRoom timeRoom;
    public Vector domain;
    public int h_val;
    
    /**
     *
     */
    public CourseSection() {
        super();
        
        course = "Unassigned";
        //instructor = "Unassigned";
        instructor = "Unassigned";
        size = 0;
        timeRoom = new TimeRoom();
    }
    
    public CourseSection(String course, String instructor, int size, TimeRoom timeRoom) {
        this.course = course;
        this.instructor = instructor;
        this.size = size;
        this.timeRoom = timeRoom;
        
        if (this.timeRoom == null) this.timeRoom = new TimeRoom();
    }
    
    public CourseSection(String course, String instructor, int size) {
        this(course,instructor,size,(new TimeRoom()));
    }
    
    public int compareTo(Object o) {
        if (o instanceof CourseSection) {
            return this.course.compareToIgnoreCase(((CourseSection)o).course);
        } else {
            return new Integer(this.hashCode()).compareTo(new Integer(o.hashCode()));
        }
    }
    
    public String getCourse() {
        return this.course;
    }
    
    public String getInstructor() {
        return this.instructor;
    }
    
    
    public int getSize() {
        return this.size;
    }
    
    public TimeRoom getTimeRoom() {
        return this.timeRoom;
    }
    
    public void setTimeRoom(TimeRoom tr) {
        this.timeRoom = tr;
    }
    
    public String toString() {
        String tr = "";
        if (timeRoom != null) {
            tr = timeRoom.toString();
        }
        return "Course: " + course +
                "\nInstructor: " + instructor +
                "\nSize: " + size +
                "\nRoom: " + tr +
                "\nh-val: " + h_val;
    }
    
    public void setXmlBean(Course c) {
        c.setBuilding(timeRoom.getBuilding());
        c.setDays(timeRoom.getSpan().getDays().toString());
        c.setEndTime(timeRoom.getSpan().getEnd().toString());
        c.setName(course);
        c.setRoom(timeRoom.getRoom());
        c.setStartTime(timeRoom.getSpan().getStart().toString());
        
        // need to build the teacher element ourselves
        org.leedberg.cotiro.CourseScheduleDocument.CourseSchedule.Course.Teacher t = c.addNewTeacher();
        t.setName(this.instructor);
        t.addNewProperties();
        
        // it's possible that we have no properties to output
        if (Scheduler.teacherProperties.get(instructor) != null) {
            
            for (Object o : Scheduler.teacherProperties.get(instructor).keySet()) {
                String propName = (String)o;
                String propVal = Scheduler.teacherProperties.get(instructor).getProperty(propName);
                org.leedberg.cotiro.CourseScheduleDocument.CourseSchedule.Course.Teacher.Properties.Property p = t.getProperties().addNewProperty();
                p.setName(propName);
                p.setValue(propVal);
                
            }
        }
        
        
        
        c.setSize(new BigInteger((new Integer(size)).toString()));
        c.setRoomSeats(new BigInteger((new Integer(timeRoom.getRoomSeats())).toString()));
    }
    
    public Object clone() {
        try {
            CourseSection c = (CourseSection)super.clone();
            c.domain = (Vector)domain.clone();
            return c;
        } catch (CloneNotSupportedException s) {
            CotiroLogic.println("Clone not supported!\n");
            return null;
        }
    }
    
    public Vector getDomain() {
        
        return this.domain;
    }
    
    public int getHval() {
        
        return this.h_val;
    }
    
    public String getUniqueID() {
        
        return this.course;
    }
    
    public Value getValue() {
        
        return this.timeRoom;
    }
    
    public void setDomain(Vector v) {
        this.domain = v;
        
    }
    
    public void setHval(int v) {
        this.h_val = v;
        
    }
    
    public void setValue(Value v) {
        this.timeRoom = (TimeRoom)v;
        
    }
}
