package org.leedberg.cotiro.college;

import org.leedberg.cotiro.TimeSpan;
import org.leedberg.cotiro.Value;

/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* This class is not at all comparable in the sense one would expect. 
   This class only needs to be sorted when ordering values, so it is 
   comparable relative to the h_val field, which is convenient for
   the intended purpose, but admittedly unintuitive. */

public class TimeRoom implements Value, Comparable
{
	protected String building;
	protected String room;	
	protected int size;
	protected TimeSpan span;
	public int h_val; // Used for soft-constraint value ordering

	/**
	 * 
	 */
	public TimeRoom() {
		super();
		building = "Unassigned";
		room = "Unassigned";	
		size = 0;	
		span = new TimeSpan();		
	}
	
	public TimeRoom ( String building, String room, int size, TimeSpan span ) {
		this.building= building;
		this.room = room;
		this.size = size;
		this.span = span;
	}
	
	public String getBuilding() {
		return this.building;
	}	
	
	public String getRoom() {
		return this.room;
	}
	
	public TimeSpan getSpan() {
		return this.span;
	}
	
	public int getRoomSeats() {
		return size;
	}
	
	public String toString() {
		return building + " " + room + ": " + span.toString() + " (" + size + " seats)";
	}	
	
	public int getSize() {
		return this.size;
	}
	
	public int compareTo(Object o) {
		if (((TimeRoom)this).h_val < ((TimeRoom)o).h_val) {
			return -1;
		}
		else if (((TimeRoom)this).h_val > ((TimeRoom)o).h_val) {
			return 1;
		}
		else {
			return 0;
		}
	}

	public int getHval() {		
		return this.h_val;
	}

	public void setHval(int hval) {
		this.h_val = hval;
		
	}
	
	
}
