package org.leedberg.cotiro.college;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import org.leedberg.cotiro.CotiroLogic;
import org.leedberg.cotiro.CourseScheduleDocument;
import org.leedberg.cotiro.CourseScheduleDocument.CourseSchedule;
import org.leedberg.cotiro.CourseScheduleDocument.CourseSchedule.Course;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
public class XmlConverter {

	public static String convertToText(File inFile) {
		String retVal = "";
		CourseScheduleDocument doc = null;

		try {
			doc = CourseScheduleDocument.Factory.parse(inFile);
		} catch (Exception e) {
			System.err.println(e.toString());
		}

		CourseSchedule schedule = doc.getCourseSchedule();
		Course[] courses = schedule.getCourseArray();

		for (int i = 0; i < courses.length; i++) {
			Course course = courses[i];

			retVal +=  System.getProperty("line.separator") + "Course: " + course.getName() +  System.getProperty("line.separator") + "Instructor: "
					+ course.getTeacher().getName() + System.getProperty("line.separator") + "Size: "
					+ course.getSize().toString()  + System.getProperty("line.separator") + "Room: "
					+ course.getBuilding() + " " + course.getRoom() + " "
					+ course.getStartTime() + " - " + course.getEndTime()
					+ " (" + course.getRoomSeats() + " seats)" + System.getProperty("line.separator");;
		}

		return retVal;
	}

	public static String convertToHtml(File inFile) {
		String retVal = "";
		CourseScheduleDocument doc = null;

		try {
			doc = CourseScheduleDocument.Factory.parse(inFile);
		} catch (Exception e) {
			System.err.println(e.toString());
		}

		CourseSchedule schedule = doc.getCourseSchedule();
		Course[] courses = schedule.getCourseArray();
		retVal = "<html><head><title>Schedule Produced By Cotiro</title></head><body><tt>";
		for (int i = 0; i < courses.length; i++) {
			Course course = courses[i];

			retVal +=  System.getProperty("line.separator")+"<p><b>Course:</b> " + course.getName()
					 + System.getProperty("line.separator")+ "<br/><b>Instructor: </b>" + course.getTeacher().getName()
					 + System.getProperty("line.separator")+ "<br/><b>Size: </b>" + course.getSize().toString()
					 + System.getProperty("line.separator") + "<br/><b>Room: </b>" + course.getBuilding() + " "
					+ course.getRoom() + " " + course.getStartTime() + " - "
					+ course.getEndTime() + " (" + course.getRoomSeats()
					+ " seats)" + System.getProperty("line.separator") + "</p>";

		}
		retVal +=  System.getProperty("line.separator") + "</tt></body></html>";
		return retVal;
	}

	/**
	 * @param args
	 */

	// XmlConverter [-xml | -html] [infile] [outfile]
	public static void main(String[] args) {
		if (args.length != 3) {
			CotiroLogic.println("Usage: XmlConverter [-text | -html] [xml_input_file] [output_file]");
			return;
		}

		String convValue = "";
		CotiroLogic.println("Reading " + args[1] + "...");
		
		if (args[0].equals("-text")) {
			CotiroLogic.println("Converting to text...");
			convValue = XmlConverter.convertToText(new File(args[1]));
		} else if (args[0].equals("-html")) {
			CotiroLogic.println("Converting to HTML...");
			convValue = XmlConverter.convertToHtml(new File(args[1]));
		} else {
			CotiroLogic.println("Unknown conversion type: " + args[1]);
			return;
		}

		try {
			CotiroLogic.println("Outputting to " + args[2]);
			PrintWriter out = new PrintWriter(new BufferedWriter(
					new FileWriter(args[2])));
			
			out.println(convValue);
			
			out.close();

		} catch (Exception e) {
			System.err.println(e.toString());
		}
		CotiroLogic.println("Done.");
	}
}
