package org.leedberg.cotiro.shiftbased;

import java.util.Iterator;
import java.util.Vector;

import org.leedberg.cotiro.CotiroLogic;
import org.leedberg.cotiro.Days;
import org.leedberg.cotiro.Time;
import org.leedberg.cotiro.TimeSpan;
import org.leedberg.cotiro.Value;
import org.leedberg.cotiro.Variable;
import org.leedberg.cotiro.CotiroDocument.Cotiro.Shifts.Shift;
import org.leedberg.cotiro.EmployeeScheduleDocument.EmployeeSchedule.Need.Employee;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
public class BuildingNeed implements Variable{
	public String buildingName;
	public String skillNeeded;
	public TimeSpan timeSpan;	
	private String uniqueId;
	private ScheduleEmployee doctor; // this is the variable we want to optimally assign
	private Vector domain;
	private int h_val;
	
	private static Long idCounter = 0L;
	
	public BuildingNeed() {
		uniqueId = String.valueOf(idCounter++);
	}
	
	public BuildingNeed(Shift need) {
		this();
		
		this.buildingName = need.getLocation();
		this.skillNeeded = need.getSkill();
		this.timeSpan = new TimeSpan(new Time(need.getStartTime()), new Time(need.getEndTime()), new Days(need.getDays()));		
		uniqueId = buildingName+"."+skillNeeded+"."+timeSpan+"."+uniqueId;
	}
	
	public void setValue(Value o) {
		if (o instanceof ScheduleEmployee) {
			doctor = (ScheduleEmployee)o;
		} else if (o == null) {
			doctor = null;
		}
		else {
			throw new IllegalArgumentException();
		}
	}
	
	public Object clone() {
		try {
			BuildingNeed c = (BuildingNeed)super.clone();		
			c.domain = (Vector)domain.clone();	
			return c;
		} catch (CloneNotSupportedException s) {
			CotiroLogic.println("Clone not supported!\n");
			return null;
		}
	}
	
	public String toString() {
		String retVal= buildingName + " needs " + skillNeeded + ": " + timeSpan + " (hval = " + h_val + ")";
		
		if (this.doctor == null) {
			retVal+= "\n\tDomain: ";
			
			Iterator i = domain.iterator();
			while (i.hasNext()) {
				retVal += ((ScheduleEmployee)i.next()).toString() + ' ';
			}
		} else {
			retVal += "\n\tDoctor = " + this.doctor;
		}
		
		return retVal;
	}

	public Vector getDomain() {
		return domain;
	}

	public void setDomain(Vector v) {
		domain = v;
		
	}

	public int getHval() {
		
		return h_val;
	}

	public void setHval(int v) {
		h_val = v;
	}

	public Value getValue() {
		
		return doctor;
	}

	public String getUniqueID() {
		return uniqueId.toString();
	}

	public void setXmlBean(org.leedberg.cotiro.EmployeeScheduleDocument.EmployeeSchedule.Need n) {
		n.setBuilding(this.buildingName);
		n.setDays(this.timeSpan.getDays().toString());
                Employee e= n.addNewEmployee();
                this.doctor.setXmlBean(e);
		//n.setEmployee(this.doctor.name);
		n.setEndTime(this.timeSpan.getEnd().toString());
		n.setSkill(this.skillNeeded);
		n.setStartTime(this.timeSpan.getStart().toString());
		
	}
}
