package org.leedberg.cotiro.shiftbased;

import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import org.leedberg.cotiro.Days;
import org.leedberg.cotiro.Time;
import org.leedberg.cotiro.TimeSpan;
import org.leedberg.cotiro.Value;
import org.leedberg.cotiro.CotiroDocument.Cotiro.Employees.Employee;
import org.leedberg.cotiro.CotiroDocument.Cotiro.Employees.Employee.Skill;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class ScheduleEmployee implements Comparable, Value{
    public String name;
    public Set<Skill> skills;
    public TimeSpan available;
    public int h_val;
    public Properties props;
    
    public ScheduleEmployee(Employee e) {
        name = e.getName();
        skills = new HashSet();
        for (Skill s : e.getSkillArray()) {
            System.err.println("Adding skill: + " + s.getName());
            skills.add(s);
        }
        available = new TimeSpan(new Time(e.getStartTime()), new Time(e.getEndTime()), new Days(e.getDays()));
        
        // do properties
        props = new Properties();
        
        if (e.getProperties() != null) {
            for (org.leedberg.cotiro.CotiroDocument.Cotiro.Employees.Employee.Properties.Property xmlprop : e.getProperties().getPropertyArray()) {
                props.setProperty(xmlprop.getName(), xmlprop.getValue());
            }
        }
    }
    public String toString() {
        String retVal = name + " (";
        
        for (Skill s : skills) {
            retVal += s.getName()+"("+s.getLimit()+")" + ' ';
        }
        return retVal + ')';
    }
    
    public int compareTo(Object o) {
        if (((ScheduleEmployee) this).h_val < ((ScheduleEmployee) o).h_val) {
            return -1;
        } else if (((ScheduleEmployee) this).h_val > ((ScheduleEmployee) o).h_val) {
            return 1;
        } else {
            return 0;
        }
    }
    public int getHval() {
        
        return h_val;
    }
    public void setHval(int hval) {
        this.h_val = hval;
        
    }
    
    public void setXmlBean(org.leedberg.cotiro.EmployeeScheduleDocument.EmployeeSchedule.Need.Employee e) {
        e.setName(this.name);
        org.leedberg.cotiro.EmployeeScheduleDocument.EmployeeSchedule.Need.Employee.Properties xmlprops = e.addNewProperties();
        
        for (Object o : this.props.keySet()) {
            String name = (String)o;
            
            org.leedberg.cotiro.EmployeeScheduleDocument.EmployeeSchedule.Need.Employee.Properties.Property xmlprop = xmlprops.addNewProperty();
            
            xmlprop.setName(name);
            xmlprop.setValue(this.props.getProperty(name));
        }
        
    }
    
}
