package org.leedberg.cotiro.shiftbased;

import java.io.File;
import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

import org.apache.xmlbeans.XmlOptions;
import org.leedberg.cotiro.Constraint;
import org.leedberg.cotiro.Constraints;
import org.leedberg.cotiro.CotiroLogic;
import org.leedberg.cotiro.SoftSetting;
import org.leedberg.cotiro.State;
import org.leedberg.cotiro.TimeSpan;
import org.leedberg.cotiro.CotiroDocument.Cotiro;
import org.leedberg.cotiro.CotiroDocument.Cotiro.Shifts.Shift;
import org.leedberg.cotiro.CotiroDocument.Cotiro.TravelTimes;
import org.leedberg.cotiro.CotiroDocument.Cotiro.Employees.Employee;
import org.leedberg.cotiro.CotiroDocument.Cotiro.Employees.Employee.Skill;
import org.leedberg.cotiro.CotiroDocument.Cotiro.TravelTimes.TravelTime;
import org.leedberg.cotiro.shiftbased.constraints.EmployeeSkillConstraint;
import org.leedberg.cotiro.shiftbased.constraints.NotSameEmployeeConstraint;
import org.leedberg.cotiro.shiftbased.constraints.ShiftLimitConstraint;
import org.leedberg.cotiro.shiftbased.constraints.TravelTimeConstraint;

/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * Created on Nov 3, 2003
 *
 */

public class Scheduler {
	// stores all of the travel times
	public static TravelTimeStore travelTimes = new TravelTimeStore();

	public Scheduler() {
		super();
	}

	public static void doSchedule(Cotiro cotiroInput, String outputName,
			String inputName, boolean doConstraints, boolean doConsistency,
			boolean doSoft, boolean doHeuristics, boolean showStates,
			boolean consoleOutput) {

		// Need to check for the relevant sections...
		if (cotiroInput.getEmployees() == null) {
			CotiroLogic.println("No employees element!");
			System.exit(-1);
		} else if (cotiroInput.getShifts() == null) {
			CotiroLogic.println("No needs element!");
			System.exit(-1);
		}
		
		// TODO: Allow user to specify # of employees per shift, and automatically
		// create correct number of variables

		// Convert the parsed data into what we need
		BuildingNeed[] bn = new BuildingNeed[cotiroInput.getShifts()
				.getShiftArray().length];
		int i = 0;
		for (Shift n : cotiroInput.getShifts().getShiftArray()) {
			bn[i++] = new BuildingNeed(n);
		}

		ScheduleEmployee[] doctors = new ScheduleEmployee[cotiroInput.getEmployees()
				.getEmployeeArray().length];
		i = 0;
		for (Employee e : cotiroInput.getEmployees().getEmployeeArray()) {
			doctors[i++] = new ScheduleEmployee(e);
		}

		if (cotiroInput.getTravelTimes() != null) {
			loadTravelTimes(cotiroInput.getTravelTimes());
		}

		State initial_state = loadInitialState(bn, doctors);
		Constraints constraints = new Constraints();
		Stack stack = new Stack();
		State endState = new State();
		CotiroLogic.println(initial_state.toString());

		if (doConstraints) {
			constraints = buildConstraints(initial_state, doSoft, doctors);
			CotiroLogic.println("Constraints: ");
			CotiroLogic.println(constraints.toString());
		}

		CotiroLogic.println("-----------------------------------------");
		CotiroLogic.println("**********PRE-PROCESSING STEPS:**********");

		if (doConsistency && doConstraints) {
			CotiroLogic.println("Forcing 1 consistency...");
			CotiroLogic.make1consistent(initial_state, constraints);
			CotiroLogic.println("done");
			initial_state.println();
		}

		if (doHeuristics && doConstraints) {
			CotiroLogic.println("Calculating h-vals...");
			CotiroLogic.calculateHvals(initial_state, constraints);
			initial_state.println();
			CotiroLogic.println("done");
		}

		// TODO: Need to test this once there are actually soft constraints in the system!!!
		if (doConstraints && doSoft) {
			CotiroLogic.println("Ordering values for soft constraints...");
			CotiroLogic.orderValues(initial_state, constraints);
			CotiroLogic.println("done");
			initial_state.println();
		}

		// At this point, we actually want to REMOVE soft constraints
		// from the list of constraints, so that states that violate them
		// are allowed. We have already done everything we need with those
		// soft states (value ordering).
		if (doConstraints && doSoft) {
			CotiroLogic.println("Removing soft constraints from system...");
			for (i = 0; i < constraints.size(); i++) {
				if (((Constraint) constraints.getConstraint(i)).isSoft) {
					constraints.removeConstraint(i);
					i--;
				}
			}
			constraints.println();

		}

		CotiroLogic.println("-----------------------------------------");

		stack.push(initial_state);

		// Back-tracking search (depth-first search)

		endState = (State) CotiroLogic.backTrackSearch(stack, constraints,
				showStates);

		if (endState != null) {
			CotiroLogic.println(endState.getAssignedVars().toString());
			// CotiroLogic.println(endState.getAssignedVars().getXmlBean()
			// .toString());

			try {
				// Sort the schedule based on course name, this should
				// help with the readability of the output file.
				//	Collections.sort(endState.getAssignedVars());

				((BuildingNeedList) endState.getAssignedVars()).getXmlBean()
						.save(new File(outputName + ".xml"),
								new XmlOptions().setUseDefaultNamespace());

				// Output text and HTML
				XmlConverter.main(new String[] { "-text", outputName + ".xml",
						outputName + ".txt" });
				XmlConverter.main(new String[] { "-html", outputName + ".xml",
						outputName + ".html" });

				if (consoleOutput) {
					CotiroLogic.println(XmlConverter.convertToText(new File(
							outputName + ".xml")));
				}
			} catch (Exception e) {
				System.err.println(e.toString());
			}

		} else {
			CotiroLogic.println("Impossible to make legal assignments!");
		}

		//CotiroLogic.println("\n**Traversed " + consideredStates + " states");

	}

	// Take the TravelTimes XMLBean, and populate the travelTimes hashmap... which is indexed by both
	// buildings, and stores a TravelTime XML Bean
	static public void loadTravelTimes(TravelTimes tts) {
		TravelTime[] ttArr = tts.getTravelTimeArray();
		CotiroLogic.println("Loading travel times...");

		for (TravelTime t : ttArr) {
			// we know that the XSD specifies that there can only be 2, no more, no less
			String building1 = t.getLocationArray(0);
			String building2 = t.getLocationArray(1);

			// travelTimes is a hashmap, where each building name maps to another HashMap.
			// the second hashmap maps 2nd building names, to an integer, which is the distance
			// between the two.   this way you can look up by either building name
			travelTimes.storeTravelTime(building1, building2, t.getTime()
					.intValue());
		}

	}

	// loadInitial state loads in the state info from the relevant files and
	// returns the
	// resulting initial state
	static private State loadInitialState(BuildingNeed[] buildingNeeds,
			ScheduleEmployee[] doctors) {
		BuildingNeedList unassignedVars = new BuildingNeedList();
		Vector availableValues = new Vector();
		BuildingNeedList assignedVars = new BuildingNeedList(); // Will be left empty

		// Initially, our unassigned variables are all of the needs that require a doctor		
		for (BuildingNeed bn : buildingNeeds) {
			unassignedVars.add(bn);
		}

		/// Initially, our values are each of the doctors that are available
		for (ScheduleEmployee d : doctors) {
			availableValues.add(d);
		}

		CotiroLogic.println(Integer.toString(availableValues.size()));
		// Start each variable with a domain of all possible values
		for (int i = 0; i < unassignedVars.size(); i++) {
			BuildingNeed bn = (BuildingNeed) unassignedVars.get(i);
			bn.setDomain((Vector) availableValues.clone());
		}
		return new State(assignedVars, unassignedVars);
	}

	// Build constraints based on implicit restraints from class times and
	// teachers,
	// and load explicit constraints from teachers file, such as teacher
	// buildings.
	private static Constraints buildConstraints(State initial_state,
			boolean doSoft, ScheduleEmployee[] employees) {
		Constraints constraints = new Constraints();
		BuildingNeedList list = (BuildingNeedList) initial_state
				.getUnassignedVars();

		// *A doctor's skill set MUST contain the building need's skill

		// For each buildingNeed, create a DoctorSkillConstraint
		Iterator i = list.iterator();
		while (i.hasNext()) {
			BuildingNeed bn = (BuildingNeed) i.next();

			Constraint c = new EmployeeSkillConstraint(bn.getUniqueID(),
					bn.skillNeeded, new SoftSetting(doSoft, true, false));
			constraints.addConstraint(c);
		}

		// *A doctor MUST NOT be assigned to two overlapping timespans
		// What we really want is a constraint between each pair of overlapping buildingNeeds, which 
		// says that they cannot be assigned the same doctor
		i = list.iterator();
		while (i.hasNext()) {
			BuildingNeed bn1 = (BuildingNeed) i.next();
			TimeSpan ts1 = bn1.timeSpan;

			// Loop over all other building need timespans...
			Iterator j = list.iterator();
			while (j.hasNext()) {
				BuildingNeed bn2 = (BuildingNeed) j.next();
				TimeSpan ts2 = bn2.timeSpan;
				if (ts1.overlap(ts2)
						&& !bn1.getUniqueID().equals(bn2.getUniqueID())) {
					// They overlap, need to create a constraint
					Constraint c = new NotSameEmployeeConstraint(bn1
							.getUniqueID(), bn2.getUniqueID(), new SoftSetting(
							doSoft, true, false));
					constraints.addConstraint(c);
				}
			}

		}

		// *If two timespans don't overlap and are in different buildings, we have to make sure
		// that if a doctor is assigned to both, the timespans MUST be far enough to allow for travel
		i = list.iterator();
		while (i.hasNext()) {
			BuildingNeed bn1 = (BuildingNeed) i.next();
			Iterator j = list.iterator();

			while (j.hasNext()) {
				BuildingNeed bn2 = (BuildingNeed) j.next();
				if ((!bn1.buildingName.equals(bn2.buildingName))
						&& (!bn1.timeSpan.overlap(bn2.timeSpan))) {

					// they don't overlap, so the same doctor may be assigned to both,
					// and they aren't the same building, so travel would be an issue
					TravelTimeConstraint tt = null;
					// need to figure out which one is earlier and which one is later
					int diff = bn1.timeSpan.getStart().minus(
							bn2.timeSpan.getStart());

					// if diff is positive, bn1 is later, else bn2 is later
					if (diff >= 0) {
						tt = new TravelTimeConstraint(bn2.getUniqueID(), bn1
								.getUniqueID(), new SoftSetting(doSoft, true,
								false));
					} else {
						tt = new TravelTimeConstraint(bn1.getUniqueID(), bn2
								.getUniqueID(), new SoftSetting(doSoft, true,
								false));
					}
					constraints.addConstraint(tt);
				}
			}
		}

                // * If an employee has a limit specified on a particular skill, we should
                //   create a constraint to check for that limit
                for (ScheduleEmployee e : employees ) {
                    for (Skill s : e.skills) {
                        if (s.getLimit() != null) {
                            // A limit exists
                            ShiftLimitConstraint slc = new ShiftLimitConstraint(e.name, s.getName(), s.getLimit().intValue(), new SoftSetting(doSoft, true,
								false));
                            constraints.addConstraint(slc);
                        }
                    }
                }
                
		// TODO: We have several constraints to generate here:
		// *All assignments for a particular doctor on a particular day SHOULD 
		//           be for the same building 
		// * MUST allow a lunch hour for each doctor

		// *Each employee should have a maximum number of hours they can work each week

		// *Employee should only be scheduled during their available time
		//    we shouldn't require available time to be defined, though

		return constraints;
	}

}
