package org.leedberg.cotiro.shiftbased;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.leedberg.cotiro.CotiroLogic;
import org.leedberg.cotiro.EmployeeScheduleDocument;
import org.leedberg.cotiro.EmployeeScheduleDocument.EmployeeSchedule;
import org.leedberg.cotiro.EmployeeScheduleDocument.EmployeeSchedule.Need;

/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

public class XmlConverter {

	public static String convertToText(File inFile) {
		String retVal = "";
		EmployeeScheduleDocument doc = null;

		try {
			doc = EmployeeScheduleDocument.Factory.parse(inFile);
		} catch (Exception e) {
			System.err.println(e.toString());
		}

		EmployeeSchedule schedule = doc.getEmployeeSchedule();
		Need[] needs = schedule.getNeedArray();

		// We want to print out a schedule, organized by employee
		// Maintain a hashMap of employees -> Vectors of needs
		HashMap employees = new HashMap();
		
		for (Need need : needs) {
			Vector v = (Vector)employees.get(need.getEmployee().getName());
			
			if (v == null) {
				// first entry for this employee
				v = new Vector();
				employees.put(need.getEmployee().getName(), v);
			}
			
			v.add(need);
		}
		
		// Now, iterate over all employees in the map, and output a section for them
		Iterator i = employees.keySet().iterator();
		
		while (i.hasNext()) {
			String emp = (String)i.next();
			Vector v = (Vector)employees.get(emp);
			
			retVal += System.getProperty("line.separator") + "Employee Name: " + emp 
				+ System.getProperty("line.separator") + System.getProperty("line.separator") 
				+ "Assignments: " + System.getProperty("line.separator");
			
			Iterator j = v.iterator();
			while (j.hasNext()) {
				Need need = (Need)j.next();
				
				retVal += "Location: " + need.getBuilding() + System.getProperty("line.separator")
					+ "Time: " + need.getStartTime() + " - " + need.getEndTime() + " " + need.getDays() + System.getProperty("line.separator")
					+ "Skill needed: " + need.getSkill() + System.getProperty("line.separator");
			}
			retVal += "----";
		}
	

		return retVal;
	}
	public static String convertToHtml(File inFile) {
		String retVal = "";
		retVal = "<html><head><title>Schedule Produced By " + CotiroLogic.version + "</title></head><body><tt>";
		EmployeeScheduleDocument doc = null;

		try {
			doc = EmployeeScheduleDocument.Factory.parse(inFile);
		} catch (Exception e) {
			System.err.println(e.toString());
		}

		EmployeeSchedule schedule = doc.getEmployeeSchedule();
		Need[] needs = schedule.getNeedArray();

		// We want to print out a schedule, organized by employee
		// Maintain a hashMap of employees -> Vectors of needs
		HashMap employees = new HashMap();
		
		for (Need need : needs) {
			Vector v = (Vector)employees.get(need.getEmployee().getName());
			
			if (v == null) {
				// first entry for this employee
				v = new Vector();
				employees.put(need.getEmployee().getName(), v);
			}
			
			v.add(need);
		}
		
		// Now, iterate over all employees in the map, and output a section for them
		Iterator i = employees.keySet().iterator();
		
		while (i.hasNext()) {
			String emp = (String)i.next();
			Vector v = (Vector)employees.get(emp);
			
			retVal += System.getProperty("line.separator") + "<p><b>Employee Name: " + emp + "</b></p>" 
				+ System.getProperty("line.separator") + System.getProperty("line.separator") 
				+ "<p><i>Assignments: </i></p>" + System.getProperty("line.separator");
			
			Iterator j = v.iterator();
			while (j.hasNext()) {
				Need need = (Need)j.next();
				
				retVal += "<p>Location: " + need.getBuilding() + "<br/>" + System.getProperty("line.separator")
					+ "Time: " + need.getStartTime() + " - " + need.getEndTime() + " " + need.getDays() + "<br/>" + System.getProperty("line.separator")
					+ "Skill needed: " + need.getSkill() + "</p>" + System.getProperty("line.separator");
			}
			retVal += "<hr>";
		}
	
		retVal += "<small>Schedule produced by " + CotiroLogic.version + "</small></body></html>";
		return retVal;
	}

	/**
	 * @param args
	 */

	// XmlConverter [-xml | -html] [infile] [outfile]
	public static void main(String[] args) {
		if (args.length != 3) {
			System.out
					.println("Usage: XmlConverter [-text | -html] [xml_input_file] [output_file]");
			return;
		}

		String convValue = "";
		CotiroLogic.println("Reading " + args[1] + "...");
		
		if (args[0].equals("-text")) {
			CotiroLogic.println("Converting to text...");
			convValue = XmlConverter.convertToText(new File(args[1]));
		} else if (args[0].equals("-html")) {
			CotiroLogic.println("Converting to HTML...");
			convValue = XmlConverter.convertToHtml(new File(args[1]));
		} else {
			CotiroLogic.println("Unknown conversion type: " + args[1]);
			return;
		}

		try {
			CotiroLogic.println("Outputting to " + args[2]);
			PrintWriter out = new PrintWriter(new BufferedWriter(
					new FileWriter(args[2])));
			
			out.println(convValue);
			
			out.close();

		} catch (Exception e) {
			System.err.println(e.toString());
		}
		CotiroLogic.println("Done.");
	}
}
