package org.leedberg.cotiro.shiftbased.constraints;

import org.leedberg.cotiro.Constraint;
import org.leedberg.cotiro.SoftSetting;
import org.leedberg.cotiro.Time;
import org.leedberg.cotiro.TimeSpan;
import org.leedberg.cotiro.VariableList;
import org.leedberg.cotiro.shiftbased.BuildingNeed;
import org.leedberg.cotiro.shiftbased.Scheduler;
/*
Cotiro 0.6.5
College Time and Room Scheduler Using Constraint Satisfaction
Copyright (C) 2003 - 2007, Gregory G. Leedberg

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
public class TravelTimeConstraint extends Constraint {

	public TravelTimeConstraint(String earlier_course, String later_course,
			SoftSetting soft) {
		super(earlier_course, later_course, soft.useSoft(false));
	}

	@Override
	public boolean satisfied(VariableList vars) {
		// Look up both course1 and course2..
		BuildingNeed bn1 = (BuildingNeed) vars.getVariable((String) this.A);
		BuildingNeed bn2 = (BuildingNeed) vars.getVariable((String) this.B);
                

		// ... if either does not exist, we are satisfied
		if (bn1 == null || bn2 == null) {

			return true;
		} else if (bn1.getValue() == null || bn2.getValue() == null) {
			// one or the other does not have an employee assigned, no
			// possible conflict

			return true;
		}
		else if (bn1.getValue().equals(bn2.getValue())){
			// these both are non-null, and have the same employee assigned
			// we want to make sure that the difference between the start time of the bn2 and the
			// end time of bn1 is at least the necessary amount
			TimeSpan later_span = bn2.timeSpan;
			Time startOfLaterSpan = later_span.getStart();
			TimeSpan early_span = bn1.timeSpan;
			Time endOfEarlySpan = early_span.getEnd();
			int difference = startOfLaterSpan.minus(endOfEarlySpan);

		
			int requiredTime = Scheduler.travelTimes.getTravelTime(bn1.buildingName, bn2.buildingName);

			return difference >= requiredTime;
			
			

		} else {
			// they don't have the same employee, so no problem with travel
			return true;
		}
	
	}

	@Override
	public String toString() {
		return "TimeTravel constraint between " + A + " and " + B;
	}

}
